@@ -25,8 +25,10 @@
 
 #include <config.h>
 #include <glib.h>
+#include <glib/gstdio.h>
 
 #include <string.h>
+#include <strings.h>
 #include <stdio.h>
 #include <time.h>
 #include <string.h>
@@ -34,13 +36,11 @@
 #include <stdlib.h>	/* atoi() */
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <ctype.h> /* tolower() */
 
 #include "gnome-i18nP.h"
 #include "gnome-util.h"
 #include "gnome-config.h"
 
-
 #if !defined getc_unlocked && !defined HAVE_GETC_UNLOCKED
 # define getc_unlocked(fp) getc (fp)
 #endif
@@ -99,12 +99,12 @@ typedef struct TProfile {
 
 static GSList *prefix_list = NULL;
 
-static TProfile *Current = 0;
+static TProfile *Current = NULL;
 
 /*
  * This one keeps track of all of the opened files
  */
-static TProfile *Base = 0;
+static TProfile *Base = NULL;
 
 static char *
 config_concat_dir_and_key (const char *dir, const char *key)
@@ -120,7 +120,7 @@ config_concat_dir_and_key (const char *dir, const char *key)
 		return g_strconcat (dir, key, NULL);
 }
 
-/* The `release_path' and `parsed_path' routines are inside the
+/* The `release_path' and `parse_path' routines are inside the
    following file.  It is in a separate file to allow the test-suite
    to get at it, without needing to export special symbols.
 
@@ -152,8 +152,8 @@ free_sections (TSecHeader *p)
 	free_sections (p->link);
 	free_keys (p->keys);
 	g_free (p->section_name);
-	p->link = 0;
-	p->keys = 0;
+	p->link = NULL;
+	p->keys = NULL;
 	g_free (p);
 }
 
@@ -283,7 +283,7 @@ escape_string_and_dup (char *s)
 	}
 	return_value = p = (char *) g_malloc (len + 1);
 	if (!return_value)
-		return 0;
+		return NULL;
 	do {
 		switch (*s){
 		case '\n':
@@ -310,12 +310,12 @@ load (const char *file)
 {
 	FILE *f;
 	int state;
-	TSecHeader *SecHeader = 0;
+	TSecHeader *SecHeader = NULL;
 	char CharBuffer [STRSIZE];
 	char *next = "";		/* Not needed */
 	int c;
 	
-	if ((f = fopen (file, "r"))==NULL)
+	if ((f = g_fopen (file, "r"))==NULL)
 		return NULL;
 	
 	state = FirstBrace;
@@ -363,7 +363,7 @@ load (const char *file)
 				temp = SecHeader;
 				SecHeader = (TSecHeader *) g_malloc (sizeof (TSecHeader));
 				SecHeader->link = temp;
-				SecHeader->keys = 0;
+				SecHeader->keys = NULL;
 				state = OnSecHeader;
 				next = CharBuffer;
 				break;
@@ -487,7 +487,7 @@ access_config (access_type mode, const char *section_name,
 		if (mode == SET){
 			new_key (section, key_name, def);
 			Current->written_to = TRUE;
-			return 0;
+			return NULL;
 		}
 	}
     
@@ -495,7 +495,7 @@ access_config (access_type mode, const char *section_name,
 	if ((mode == SET) && def){
 		section = (TSecHeader *) g_malloc (sizeof (TSecHeader));
 		section->section_name = g_strdup (section_name);
-		section->keys = 0;
+		section->keys = NULL;
 		new_key (section, key_name, def);
 		section->link = Current->section;
 		Current->section = section;
@@ -506,10 +506,10 @@ access_config (access_type mode, const char *section_name,
 	return def;
 }
 
-/* an extended version of access_config for looking up values in ~/.gnome.
+/* an extended version of access_config for looking up values in ~/.gnome2.
  * For writes it falls through to the standard behaviour.
  * For lookups, it first checks for the value in
- * $(datadir)/gnome/config-override, and if it isn't there in ~/.gnome,
+ * $(datadir)/gnome/config-override, and if it isn't there in ~/.gnome2,
  * then checks $(datadir)/gnome/config, and as a last fallback uses def.
  * This gives system administrators high level control over the default
  * configuration values for GNOME
@@ -656,33 +656,22 @@ dump_sections (FILE *profile, TSecHeader *p)
 static gint
 check_path(char *path, mode_t newmode)
 {
-	gchar *dirpath;
-	gchar *p;
-	GString *newpath;
+	gchar *dir, *root, *p, *cur;
+	GString *newpath = NULL;
 	struct stat s;
-	char *tokp;
 
 	g_return_val_if_fail (path != NULL, FALSE);
 
-	if(strchr(path,'/')==NULL)
-		return FALSE;
-
-	dirpath = strcpy (g_alloca (strlen (path) + 1), path);
-	g_return_val_if_fail (dirpath != NULL, FALSE);
-
-	if (*dirpath == '\0')
-		return FALSE;
-
 	/*not absolute, we refuse to work*/
-	if (dirpath[0] != '/')
+	if (!g_path_is_absolute(path))
 		return FALSE;
 
-	p = strrchr(dirpath,'/');
-		*p='\0';
+	dir = g_path_get_dirname (path);
 
 	/*special case if directory exists, this is probably gonna happen
 	  a lot so we don't want to go though checking it part by part*/
-	if (stat(dirpath, &s) == 0) {
+	if (stat(dir, &s) == 0) {
+		g_free(dir);
 		/*check if a directory*/
 		if (!S_ISDIR(s.st_mode))
 			return FALSE;
@@ -691,35 +680,53 @@ check_path(char *path, mode_t newmode)
 	}
 
 
-	/*skip leading '/'*/
-	p = dirpath;
-	while(*p == '/')
-		p++;
+	p = (gchar *) g_path_skip_root(dir);
+	root = g_strndup(dir, p-dir);
 
-	p = strtok_r(p, "/", &tokp);
-	newpath = g_string_new("");
-	do {
-		newpath = g_string_append_c(newpath,'/');
+	cur = p;
+	while ((p = cur)) {
+	       	cur = strchr (p, G_DIR_SEPARATOR);
+#ifdef G_OS_WIN32
+		{
+			gchar *cur2 = strchr (p, '/');
+			if (cur == NULL || (cur2 != NULL && cur2 < cur))
+				cur = cur2;
+		}
+#endif
+		if (cur) {
+			*cur = '\0';
+			cur++;
+		}
+		
+		if (newpath == NULL)
+			newpath = g_string_new(root);
+		else
+			newpath = g_string_append_c(newpath, G_DIR_SEPARATOR);
 		newpath = g_string_append(newpath,p);
 		if(stat(newpath->str,&s)==0) {
 			/*check if a directory*/
 			if(!S_ISDIR(s.st_mode)) {
 				g_string_free(newpath,TRUE);
+				g_free(root);
+				g_free(dir);
 				return FALSE;
 			}
 		} else {
 			/*we couldn't stat it .. let's try making the
 			  directory*/
-			if(mkdir(newpath->str,newmode)!=0) {
+			if(g_mkdir(newpath->str,newmode)!=0) {
 				/*error, return false*/
 				g_string_free(newpath,TRUE);
+				g_free(root);
+				g_free(dir);
 				return FALSE;
 			}
 		}
-
-	} while ((p = strtok_r(NULL, "/", &tokp)) != NULL);
+	}
 
 	g_string_free(newpath,TRUE);
+	g_free(root);
+	g_free(dir);
 
 	return TRUE;
 }
@@ -758,7 +765,7 @@ dump_profile (TProfile *p, gboolean one_only)
 		if(p->to_be_deleted) {
 			/*remove the file and remove all it's ramaints
 			  from memory*/
-			unlink(p->filename);
+			g_unlink(p->filename);
 			/* this already must have been true */
 			/*p->section = 0;*/
 			p->filename [0] = '\0';
@@ -767,7 +774,7 @@ dump_profile (TProfile *p, gboolean one_only)
 			if(p==Current)
 				Current = NULL;
 		} else if (check_path(p->filename,0755) &&
-		    (profile = fopen (p->filename, "w")) != NULL){
+		    (profile = g_fopen (p->filename, "w")) != NULL){
 			dump_sections (profile, p->section);
 			fclose (profile);
 		} else {
@@ -788,10 +795,10 @@ dump_profile (TProfile *p, gboolean one_only)
  * Writes all of the information modified by gnome-config to the
  * disk.
  *
- * Note: the gnome_config code does not write anything to the
+ * Note: the gnome-config code does not write anything to the
  * configuration files until this routine is actually invoked.
  *
- * Returns: %TRUE if everything went well.  %FALSE if any file
+ * Returns: %TRUE if everything went well. %FALSE if any file
  * could not be written to disk.
  */
 gboolean 
@@ -810,9 +817,13 @@ gnome_config_sync (void)
  * Writes all of the information modified by gnome-config to the
  * disk for the given file.
  *
- * Note: the gnome_config code does not write anything to the
- * configuration files until this routine or gnome_config_sync 
+ * Note: the gnome-config code does not write anything to the
+ * configuration files until this routine or gnome_config_sync()
  * is actually invoked.
+ *
+ * Returns: %TRUE if everything went well, %FALSE if the file
+ * could not be written to for some reason.  %FALSE is only returned
+ * when a write was actually attempted and failed.
  */
 /**
  * gnome_config_private_sync_file:
@@ -821,8 +832,8 @@ gnome_config_sync (void)
  * Writes all of the information modified by gnome-config to the
  * disk for the given private file.
  *
- * Note: the gnome_config code does not write anything to the
- * configuration files until this routine or gnome_config_sync 
+ * Note: the gnome-config code does not write anything to the
+ * configuration files until this routine or gnome_config_sync()
  * is actually invoked.
  *
  * Returns: %TRUE if everything went well, %FALSE if the file
@@ -866,7 +877,7 @@ gnome_config_sync_file_ (char *path, gboolean priv)
  * Cleans up the configuration file specified by @path from any
  * configuration information.
  *
- * Changes will take place after #gnome_config_sync has been invoked.
+ * Changes will take place after gnome_config_sync() has been invoked.
  */
 /**
  * gnome_config_private_clean_file:
@@ -875,7 +886,7 @@ gnome_config_sync_file_ (char *path, gboolean priv)
  * Cleans up the private configuration file specified by @path from
  * any configuration information.
  *
- * Changes will take place after #gnome_config_sync has been invoked.
+ * Changes will take place after gnome_config_sync() has been invoked.
  */
 void 
 gnome_config_clean_file_ (const char *path, gboolean priv)
@@ -913,7 +924,7 @@ gnome_config_clean_file_ (const char *path, gboolean priv)
  *
  * Releases any memory resources that were allocated from accessing
  * the configuration file in @path.  Changes will take place after
- * #gnome_config_sync has been invoked
+ * gnome_config_sync() has been invoked
  */
 /**
  * gnome_config_private_drop_file:
@@ -964,9 +975,9 @@ gnome_config_drop_file_ (const char *path, gboolean priv)
  * Creates an iterator handle that can be used to
  * iterate over the keys in a section in a gnome configuration
  * file.  @path must refer to a section.  The returned value
- * can be used as an iterator for #gnome_config_iterator_next().
+ * can be used as an iterator for gnome_config_iterator_next().
  *
- * Returns the iterator handle.
+ * Returns: The iterator handle.
  */
 /**
  * gnome_config_private_init_iterator:
@@ -975,9 +986,9 @@ gnome_config_drop_file_ (const char *path, gboolean priv)
  * Creates an iterator handle that can be used to
  * iterate over the keys in a section in a private gnome configuration
  * file.  @path must refer to a section.  The returned value
- * can be used as an iterator for #gnome_config_iterator_next().
+ * can be used as an iterator for gnome_config_iterator_next().
  *
- * Returns the iterator handle.
+ * Returns: The iterator handle.
  */
 void *
 gnome_config_init_iterator_ (const char *path, gboolean priv)
@@ -1022,7 +1033,7 @@ gnome_config_init_iterator_ (const char *path, gboolean priv)
 		return iter;
 	}
 	release_path (pp);
-	return 0;
+	return NULL;
 }
 
 
@@ -1033,9 +1044,9 @@ gnome_config_init_iterator_ (const char *path, gboolean priv)
  * Creates an iterator handle that can be used to iterate over the
  * sections in a gnome configuration file.  @path must refer to a
  * gnome configuration file.  The returned value can be used as an
- * iterator for #gnome_config_iterator_next().
+ * iterator for gnome_config_iterator_next().
  *
- * Returns the iterator handle.
+ * Returns: The iterator handle.
  */
 /**
  * gnome_config_private_init_iterator_sections:
@@ -1044,9 +1055,9 @@ gnome_config_init_iterator_ (const char *path, gboolean priv)
  * Creates an iterator handle that can be used to iterate over the
  * sections in a private gnome configuration file.  @path must refer to a
  * gnome configuration file.  The returned value can be used as an
- * iterator for #gnome_config_iterator_next().
+ * iterator for gnome_config_iterator_next().
  *
- * Returns the iterator handle.
+ * Returns: The iterator handle.
  */
 void *
 gnome_config_init_iterator_sections_ (const char *path, gboolean priv)
@@ -1094,15 +1105,11 @@ gnome_config_init_iterator_sections_ (const char *path, gboolean priv)
  * @key:   Address where the key gets stored.
  * @value: Address where the value gets stored.
  *
- *
- * Returns a new iterator handle.
- *
  * If @key is non-NULL, then @key will point to a g_malloc()ed region that
  * holds the key.
  *
  * If @value is non-NULL, then @value will point to a g_malloc()ed region that
  * holds the key.
- *
  */
 void *
 gnome_config_iterator_next (void *iterator_handle, char **key, char **value)
@@ -1135,7 +1142,7 @@ gnome_config_iterator_next (void *iterator_handle, char **key, char **value)
 			return iter;
 		} else {
 			g_free (iter);
-			return 0;
+			return NULL;
 		}
 	} else {
 		TSecHeader *section;
@@ -1149,26 +1156,26 @@ gnome_config_iterator_next (void *iterator_handle, char **key, char **value)
 			return iter;
 		} else {
 			g_free (iter);
-			return 0;
+			return NULL; 
 		}
 	}
 }
 
 /**
  * gnome_config_clean_section:
- * @path: a gnome configuration path to a section.
+ * @path: A gnome configuration path to a section.
  *
  * Cleans up the section specified by @path from any
  * configuration information.  Changes will only take place
- * after #gnome_config_sync has been invoked.
+ * after gnome_config_sync() has been invoked.
  */
 /**
  * gnome_config_private_clean_section:
- * @path: a gnome configuration path to a section.
+ * @path: A gnome configuration path to a section.
  *
  * Cleans up the section specified by @path in a private file from any
  * configuration information.  Changes will only take place after
- * #gnome_config_sync has been invoked.
+ * gnome_config_sync() has been invoked.
  */
 void 
 gnome_config_clean_section_ (const char *path, gboolean priv)
@@ -1214,20 +1221,20 @@ gnome_config_clean_section_ (const char *path, gboolean priv)
 
 /**
  * gnome_config_clean_key:
- * @path: a gnome configuration path to a key.
+ * @path: A gnome configuration path to a key.
  *
  * Removes the definition for the key on a gnome configuration file.
  *
- * Changes will take place after #gnome_config_sync has been invoked.
+ * Changes will take place after gnome_config_sync() has been invoked.
  */
 /**
  * gnome_config_private_clean_key:
- * @path: a gnome configuration path to a key.
+ * @path: A gnome configuration path to a key.
  *
  * Removes the definition for the key on a private gnome configuration
  * file.
  *
- * Changes will take place after #gnome_config_sync has been invoked.
+ * Changes will take place after gnome_config_sync() has been invoked.
  */
 void 
 gnome_config_clean_key_ (const char *path, gboolean priv)
@@ -1278,7 +1285,7 @@ gnome_config_clean_key_ (const char *path, gboolean priv)
  * Queries the gnome configuration file for the presence
  * of the section specified in @path.
  *
- * Returns TRUE if the section exists.  FALSE otherwise.
+ * Returns: %TRUE if the section exists, %FALSE otherwise.
  */
 /**
  * gnome_config_private_has_section:
@@ -1287,7 +1294,7 @@ gnome_config_clean_key_ (const char *path, gboolean priv)
  * Queries the private gnome configuration file for the presence
  * of the section specified in @path.
  *
- * Returns TRUE if the section exists.  FALSE otherwise.
+ * Returns: %TRUE if the section exists, %FALSE otherwise.
  */
 gboolean 
 gnome_config_has_section_ (const char *path, gboolean priv)
@@ -1334,7 +1341,7 @@ gnome_config_has_section_ (const char *path, gboolean priv)
  * gnome_config_drop_all:
  *
  * Drops any information cached in memory that was fetched with
- * gnome config.  Any pending information that has not been
+ * gnome config. Any pending information that has not been
  * written to disk is discarded.
  */
 void 
@@ -1349,14 +1356,18 @@ gnome_config_drop_all (void)
  * gnome_config_get_int:
  * @path: A gnome configuration path to an item.
  *
- * Retrieves & returns the value of a configuration item as an integer.
+ * Retrieves an integer value configuration item.
+ *
+ * Returns: The value of a configuration item.
  */
 /**
  * gnome_config_private_get_int:
  * @path: A gnome configuration path to an item in the user-private namespace.
  *
- * Retrieves & returns the value of a configuration item as an integer.
- * The item is retrieved from the user's private configuration storage area.
+ * Retrieves a configuration item as an int from the user's private
+ * configuration storage area.
+ *
+ * Returns: The value of a configuration item as an integer.
  */
 /**
  * gnome_config_get_int_with_default:
@@ -1364,7 +1375,10 @@ gnome_config_drop_all (void)
  * @def: A pointer to a flag that will be set if the default value for the item
  * is returned.
  *
- * Retrieves & returns the value of a configuration item as an integer.
+ * Retrieves an integer value configuration item.
+ *
+ * Returns: The value of a configuration item as an integer or @def if the
+ * configuration item does not exist.
  */
 /**
  * gnome_config_private_get_int_with_default:
@@ -1372,8 +1386,11 @@ gnome_config_drop_all (void)
  * @def: A pointer to a flag that will be set if the default value for the item
  * is returned.
  *
- * Retrieves & returns the value of a configuration item as an integer.
- * The item is retrieved from the user's private configuration storage area.
+ * Retrieves a configuration item as an int from the user's private
+ * configuration storage area.
+ *
+ * Returns: The value of a configuration item as an integer or @def if the
+ * configuration item does not exist.
  */
 gint
 gnome_config_get_int_with_default_ (const char *path, gboolean *def, gboolean priv)
@@ -1406,15 +1423,19 @@ gnome_config_get_int_with_default_ (const char *path, gboolean *def, gboolean pr
  * gnome_config_get_float:
  * @path: A gnome configuration path to an item.
  *
- * Retrieves & returns the value of a configuration item as a floating-point
- * number.
+ * Retrieves a floating-point valued configuration item.
+ *
+ * Returns: The value of a configuration item.
  */
 /**
  * gnome_config_private_get_float:
  * @path: A gnome configuration path to an item in the user-private namespace.
  *
- * Retrieves & returns the value of a configuration item as a floating-point number.
- * The item is retrieved from the user's private configuration storage area.
+ * Retrieves a configuration item from the user's private configuration storage
+ * area.
+ *
+ * Returns: The value of a configuration item as a floating-point
+ * number.
  */
 /**
  * gnome_config_get_float_with_default:
@@ -1422,7 +1443,10 @@ gnome_config_get_int_with_default_ (const char *path, gboolean *def, gboolean pr
  * @def: A pointer to a flag that will be set if the default value for the item
  * is returned.
  *
- * Retrieves & returns the value of a configuration item as a floating-point number.
+ * Retrieves a floating-point valued configuration item.
+ *
+ * Returns: The value of a configuration item as a floating-point
+ * number or @def if the configuration item does not exist.
  */
 /**
  * gnome_config_private_get_float_with_default:
@@ -1430,8 +1454,11 @@ gnome_config_get_int_with_default_ (const char *path, gboolean *def, gboolean pr
  * @def: A pointer to a flag that will be set if the default value for the item
  * is returned.
  *
- * Retrieves & returns the value of a configuration item as a floating-point number.
- * The item is retrieved from the user's private configuration storage area.
+ * Retrieves a configuration item from the user's private configuration storage
+ * area.
+ *
+ * Returns: The value of a configuration item as a floating-point
+ * number or @def if the configuration item does not exist.
  */
 gdouble
 gnome_config_get_float_with_default_ (const char *path, gboolean *def, gboolean priv)
@@ -1512,17 +1539,22 @@ get_string_with_default_from_pp_with_lang (ParsedPath *pp,
  * gnome_config_get_translated_string:
  * @path: A gnome configuration path to an item.
  *
- * Retrieves & returns the value of a configuration item as a string
- * appropriate for the current language. The returned value should be
- * g_free()'d when no longer needed.
+ * Retrieves the value of a configuration item as a string appropriate for the
+ * current language. The returned value should be freed with g_free() when no
+ * longer needed.
+ *
+ * Returns: The value of the configuration item.
  */
 /**
  * gnome_config_private_get_translated_string:
  * @path: A gnome configuration path to an item in the user-private namespace.
  *
- * Retrieves & returns the value of a configuration item as a string appropriate for the current language. The returned value should be
- * g_free()'d when no longer needed.
- * The item is retrieved from the user's private configuration storage area.
+ * Retrieves the value of a configuration item as a string appropriate for the
+ * current language. The returned value should be freed with g_free() when no
+ * longer needed.  The item is retrieved from the user's private configuration
+ * storage area.
+ *
+ * Returns: The value of the configuration item.
  */
 /**
  * gnome_config_get_translated_string_with_default:
@@ -1530,8 +1562,12 @@ get_string_with_default_from_pp_with_lang (ParsedPath *pp,
  * @def: A pointer to a flag that will be set if the default value for the item
  * is returned.
  *
- * Retrieves & returns the value of a configuration item as a string appropriate for the current language. The returned value should be
- * g_free()'d when no longer needed.
+ * Retrieves the value of a configuration item as a string appropriate for the
+ * current language. The returned value should be freed with g_free() when no
+ * longer needed.
+ *
+ * Returns: The value of the configuration item or @def if the configuration
+ * item does not exist.
  */
 /**
  * gnome_config_private_get_translated_string_with_default:
@@ -1539,9 +1575,13 @@ get_string_with_default_from_pp_with_lang (ParsedPath *pp,
  * @def: A pointer to a flag that will be set if the default value for the item
  * is returned.
  *
- * Retrieves & returns the value of a configuration item as a string appropriate for the current language. The returned value should be
- * g_free()'d when no longer needed.
- * The item is retrieved from the user's private configuration storage area.
+ * Retrieves the value of a configuration item as a string appropriate for the
+ * current language. The returned value should be freed with g_free() when no
+ * longer needed.  The item is retrieved from the user's private configuration
+ * storage area.
+ *
+ * Returns: The value of the configuration item or @def if the configuration
+ * item does not exist.
  */
 char *
 gnome_config_get_translated_string_with_default_ (const char *path,
@@ -1549,17 +1589,19 @@ gnome_config_get_translated_string_with_default_ (const char *path,
 						  gboolean priv)
 {
 	ParsedPath *pp;
-	const GList *language_list;
+	const char * const *language_list;
 	gboolean local_def = FALSE;
+	int i;
 
 	char *value= NULL;
 
-	language_list = gnome_i18n_get_language_list ("LC_MESSAGES");
+	language_list = g_get_language_names ();
 
 	pp = parse_path (path, priv);
 
-	while (!value && language_list) {
-		const char *lang= language_list->data;
+	i = 0;
+	while (!value && language_list[i] != NULL) {
+		const char *lang = language_list[i];
 
 		value = get_string_with_default_from_pp_with_lang
 			(pp, lang, &local_def, priv);
@@ -1585,7 +1627,7 @@ gnome_config_get_translated_string_with_default_ (const char *path,
 				}
 			}
 		}
-		language_list = language_list->next;
+		i++;
 	}
 
 	if (def != NULL) {
@@ -1610,18 +1652,20 @@ gnome_config_get_translated_string_with_default_ (const char *path,
  * gnome_config_get_string:
  * @path: A gnome configuration path to an item.
  *
- * Retrieves & returns the value of a configuration item as a string.
- * The returned value should be
- * g_free()'d when no longer needed.
+ * Retrieves the value of a configuration item as a string. This value should
+ * be freed with g_free() when no longer needed.
+ *
+ * Returns: The value of the configuration item as a string.
  */
 /**
  * gnome_config_private_get_string:
  * @path: A gnome configuration path to an item in the user-private namespace.
  *
- * Retrieves & returns the value of a configuration item as a
- * string. The returned value should be g_free()'d when no longer
- * needed.  The item is retrieved from the user's private
- * configuration storage area.
+ * Retrieves the value of a configuration item from the user's private
+ * configuration directory as a string. This value should be freed with
+ * g_free() when no longer needed.
+ *
+ * Returns: The value of the configuration item as a string.
  */
 /**
  * gnome_config_get_string_with_default:
@@ -1629,9 +1673,11 @@ gnome_config_get_translated_string_with_default_ (const char *path,
  * @def: A pointer to a flag that will be set if the default value for the item
  * is returned.
  *
- * Retrieves & returns the value of a configuration item as a string.
- * The returned value should be g_free()'d when no longer needed.
+ * Retrieves the value of a configuration item as a string. This value should
+ * be freed with g_free() when no longer needed.
  *
+ * Returns: The value of the configuration item as a string, or @def if the
+ * configuration key does not exist.
  */
 /**
  * gnome_config_private_get_string_with_default:
@@ -1639,11 +1685,12 @@ gnome_config_get_translated_string_with_default_ (const char *path,
  * @def: A pointer to a flag that will be set if the default value for the item
  * is returned.
  *
- * Retrieves & returns the value of a configuration item as a string.
- * The returned value should be g_free()'d when no longer needed.  The
- * item is retrieved from the user's private configuration storage
- * area.
+ * Retrieves the value of a configuration item from the user's private
+ * configuration directory as a string. This value should be freed with
+ * g_free() when no longer needed.
  *
+ * Returns: The value of the configuration item as a string, or @def if the
+ * configuration key does not exist.
  */
 char *
 gnome_config_get_string_with_default_ (const char *path, gboolean *def,
@@ -1663,14 +1710,17 @@ gnome_config_get_string_with_default_ (const char *path, gboolean *def,
  * gnome_config_get_bool:
  * @path: A gnome configuration path to an item.
  *
- * Retrieves & returns the value of a configuration item as a boolean.
+ * Retrieves a boolean configuration value.
+ *
+ * Returns: The value of a configuration item.
  */
 /**
  * gnome_config_private_get_bool:
  * @path: A gnome configuration path to an item in the user-private namespace.
  *
- * Retrieves & returns the value of a configuration item as a boolean.
- * The item is retrieved from the user's private configuration storage area.
+ * Retrieves the item from the user's private configuration storage area.
+ *
+ * Returns: The value of a configuration item as a boolean.
  */
 /**
  * gnome_config_get_bool_with_default:
@@ -1678,7 +1728,10 @@ gnome_config_get_string_with_default_ (const char *path, gboolean *def,
  * @def: A pointer to a flag that will be set if the default value for the item
  * is returned.
  *
- * Retrieves & returns the value of a configuration item as a boolean.
+ * Retrieves a boolean configuration value.
+ *
+ * Returns: The value of a configuration item, or @def if the
+ * configuration item does not exist.
  */
 /**
  * gnome_config_private_get_bool_with_default:
@@ -1686,8 +1739,10 @@ gnome_config_get_string_with_default_ (const char *path, gboolean *def,
  * @def: A pointer to a flag that will be set if the default value for the item
  * is returned.
  *
- * Retrieves & returns the value of a configuration item as a boolean.
- * The item is retrieved from the user's private configuration storage area.
+ * Retrieves the item from the user's private configuration storage area.
+ *
+ * Returns: The value of a configuration item as a boolean, or @def if the
+ * configuration item does not exist.
  */
 gboolean
 gnome_config_get_bool_with_default_ (const char *path, gboolean *def,
@@ -1711,7 +1766,7 @@ gnome_config_get_bool_with_default_ (const char *path, gboolean *def,
 		return 0;
 	}
 
-	if (tolower(*r) == 't' || tolower(*r) == 'y' || atoi(r)) {
+	if (g_ascii_tolower(*r) == 't' || g_ascii_tolower(*r) == 'y' || atoi(r)) {
 	  v = 1;
 	} else {
 	  /* If it's not true it has to be false :) */
@@ -1724,8 +1779,8 @@ gnome_config_get_bool_with_default_ (const char *path, gboolean *def,
 /**
  * gnome_config_make_vector:
  * @string: The stringified vector to decode into 'argcp' and 'argvp'
- * @argcp: Returns the number of elements in 'argvp'
- * @argvp: Returns the array of strings found in 'rr'.
+ * @argcp: Returns the number of elements in @string.
+ * @argvp: Returns the array of strings found in @string.
  *
  * Creates a new vector from a string as it stored in the config file,
  * breaks the string on spaces except if the space is escaped with a
@@ -1796,34 +1851,33 @@ gnome_config_make_vector (const char *string, int *argcp, char ***argvp)
 /**
  * gnome_config_get_vector:
  * @path: A gnome configuration path to an item.
- * @argcp: Number of elements in the vector
- * @argvp: Vector of strings
+ * @argcp: Number of elements in the vector.
+ * @argvp: Vector of strings.
  *
- * Retrieves & returns the value of a configuration item as a string array.
- * The returned value should be
- * g_free()'d when no longer needed.
+ * Retrieves the value of a configuration item as a string array.
+ * The returned vector should be freed with g_free() when no longer needed.
  */
 /**
  * gnome_config_private_get_vector:
  * @path: A gnome configuration path to an item in the user-private namespace.
- * @argcp: Number of elements in the vector
- * @argvp: Vector of strings
+ * @argcp: Number of elements in the vector.
+ * @argvp: Vector of strings.
  *
- * Retrieves & returns the value of a configuration item as a
- * string. The returned value should be g_free()'d when no longer
- * needed.  The item is retrieved from the user's private
- * configuration storage area.
+ * Retrieves the value of a configuration item as a string array.
+ * The returned vector should be freed with g_free() when no longer needed. The
+ * configuration value is retrieved from the user's private configuration
+ * storage area.
  */
 /**
  * gnome_config_get_vector_with_default:
  * @path: A gnome configuration path to an item.
- * @argcp: Number of elements in the vector
- * @argvp: Vector of strings
+ * @argcp: Number of elements in the vector.
+ * @argvp: Vector of strings.
  * @def: A pointer to a flag that will be set if the default value for the item
  * is returned.
  *
- * Retrieves & returns the value of a configuration item as a string array.
- * The returned value should be g_free()'d when no longer needed.
+ * Retrieves the value of a configuration item as a string array.
+ * The returned vector should be freed with g_free() when no longer needed.
  *
  */
 /**
@@ -1834,11 +1888,10 @@ gnome_config_make_vector (const char *string, int *argcp, char ***argvp)
  * @def: A pointer to a flag that will be set if the default value for the item
  * is returned.
  *
- * Retrieves & returns the value of a configuration item as a string array.
- * The returned value should be g_free()'d when no longer needed.  The
- * item is retrieved from the user's private configuration storage
- * area.
- *
+ * Retrieves the value of a configuration item as a string array.
+ * The returned vector should be freed with g_free() when no longer needed. The
+ * configuration value is retrieved from the user's private configuration
+ * storage area.
  */
 void
 gnome_config_get_vector_with_default_ (const char *path, int *argcp,
@@ -1865,24 +1918,32 @@ gnome_config_get_vector_with_default_ (const char *path, int *argcp,
 
 /**
  * gnome_config_set_translated_string:
- * @path: a gnome configuration path to a key
- * @value: a string value to set.
+ * @path: A gnome configuration path to a key.
+ * @value: A string value to set.
  * 
- * Stores the string value @new_value in the file/section/key defined
+ * Stores the string value @value in the file/section/key defined
  * by the @path on the proper section for the current language set by
  * by the user.
  */
+/**
+ * gnome_config_private_set_translated_string:
+ * @path: A gnome configuration path to a key.
+ * @new_value: A string value to set.
+ * 
+ * Stores the string value @new_value in the file/section/key defined by the
+ * @path on the proper section for the current language set by by the user.
+ * The configuration value is stored in the user's private storage area.
+ */
 void
 gnome_config_set_translated_string_ (const char *path, const char *value,
 				     gboolean priv)
 {
-	const GList *language_list;
+	const char * const *language_list;
 	const char *lang;
 	char *tkey;
 
-	language_list = gnome_i18n_get_language_list("LC_MESSAGES");
-
-	lang= language_list ? language_list->data : NULL;
+	language_list = g_get_language_names ();
+	lang = language_list[0];
 
 	if (lang && (strcmp (lang, "C") != 0)) {
 		tkey = g_strconcat (path, "[", lang, "]", NULL);
@@ -1894,11 +1955,20 @@ gnome_config_set_translated_string_ (const char *path, const char *value,
 
 /**
  * gnome_config_set_string:
- * @path: a gnome configuration path to a key
- * @new_value: a string value to set.
+ * @path: A gnome configuration path to a key.
+ * @new_value: A string value to set.
  *
  * Stores the string value @new_value in the file/section/key
- * defined by the @path
+ * defined by the @path.
+ */
+/**
+ * gnome_config_private_set_string:
+ * @path: A gnome configuration path to a key.
+ * @new_value: A string value to set.
+ *
+ * Stores the string value @new_value in the file/section/key
+ * defined by the @path. The configuration value is stored in the user's
+ * private storage area.
  */
 void
 gnome_config_set_string_ (const char *path, const char *new_value, gboolean priv)
@@ -1914,11 +1984,20 @@ gnome_config_set_string_ (const char *path, const char *new_value, gboolean priv
 
 /**
  * gnome_config_set_int:
- * @path: a gnome configuration path to a key
- * @new_value: a int value to set.
+ * @path: A gnome configuration path to a key.
+ * @new_value: A int value to set.
  *
  * Stores the integer value @new_value in the file/section/key
- * defined by the @path
+ * defined by the @path.
+ */
+/**
+ * gnome_config_private_set_int:
+ * @path: A gnome configuration path to a key.
+ * @new_value: A int value to set.
+ *
+ * Stores the integer value @new_value in the file/section/key
+ * defined by the @path. The value is stored in the user's private
+ * configuration storage area.
  */
 void
 gnome_config_set_int_ (const char *path, int new_value, gboolean priv)
@@ -1936,11 +2015,20 @@ gnome_config_set_int_ (const char *path, int new_value, gboolean priv)
 
 /**
  * gnome_config_set_float:
- * @path: a gnome configuration path to a key
- * @new_value: a double value to set.
+ * @path: A gnome configuration path to a key.
+ * @new_value: A double value to set.
  *
  * Stores the double value @new_value in the file/section/key
- * defined by the @path
+ * defined by the @path.
+ */
+/**
+ * gnome_config_private_set_float:
+ * @path: A gnome configuration path to a key.
+ * @new_value: A double value to set.
+ *
+ * Stores the double value @new_value in the file/section/key
+ * defined by the @path. The value is stored in the user's private
+ * configuration storage area.
  */
 void
 gnome_config_set_float_ (const char *path, gdouble new_value, gboolean priv)
@@ -1963,12 +2051,20 @@ gnome_config_set_float_ (const char *path, gdouble new_value, gboolean priv)
 
 /**
  * gnome_config_set_bool:
- * @path: a gnome configuration path to a key
- * @new_value: a boolean value to set
+ * @path: A gnome configuration path to a key.
+ * @new_value: A boolean value to set.
  *
  * Stores boolean value @new_value in the file/section/key defined by
  * @path.
  */
+/**
+ * gnome_config_private_set_bool:
+ * @path: A gnome configuration path to a key.
+ * @new_value: A boolean value to set.
+ *
+ * Stores boolean value @new_value in the file/section/key defined by @path.
+ * The value is stored in the user's private configuration storage area.
+ */
 void
 gnome_config_set_bool_ (const char *path, gboolean new_value, gboolean priv)
 {
@@ -1987,9 +2083,10 @@ gnome_config_set_bool_ (const char *path, gboolean new_value, gboolean priv)
  * @argv: An array of strings.
  *
  * This routine returns the the strings in the array contactenated by
- * spaces.
+ * spaces. The return value should be freed with g_free() when it is no longer
+ * required.
  *
- * Returns: a g_malloc()ed string with the concatenation results.
+ * Returns: A string with the concatenation results.
  */
 char *
 gnome_config_assemble_vector (int argc, const char *const argv [])
@@ -2023,13 +2120,22 @@ gnome_config_assemble_vector (int argc, const char *const argv [])
 
 /**
  * gnome_config_set_vector:
- * @path: a gnome configuration path to a key
- * @argc: the number of elements in @argv
- * @argv: a string array holding the data to store.
+ * @path: A gnome configuration path to a key.
+ * @argc: The number of elements in @argv.
+ * @argv: A string array holding the data to store.
  *
  * Stores vector @argv in the file/section/key defined by
  * @path.
  */
+/**
+ * gnome_config_private_set_vector:
+ * @path: A gnome configuration path to a key.
+ * @argc: The number of elements in @argv.
+ * @argv: A string array holding the data to store.
+ *
+ * Stores vector @argv in the file/section/key defined by @path. The
+ * configuration value is set in the user's private storage area.
+ */
 void
 gnome_config_set_vector_ (const char *path, int argc,
 			  const char *const argv[],
@@ -2047,7 +2153,7 @@ gnome_config_set_vector_ (const char *path, int argc,
 
 /**
  * gnome_config_push_prefix:
- * @path: a gnome configuration path prefix
+ * @path: A gnome configuration path prefix.
  *
  * @path is a prefix that will be prepended automatically to any
  * non-absolute configuration path in gnome config.
