@@ -39,6 +39,7 @@
 
 #ifdef HAVE_ESD
 static char *esound_hostname = NULL;
+static gboolean esound_hostname_null_ok = FALSE;
 static int gnome_sound_connection = -1;
 #endif
 
@@ -100,7 +101,7 @@ gnome_sound_sample_load_wav(const char *file)
   f = fopen (file, "r");
   if (!f)
     return NULL;
-  s = g_malloc (sizeof (GnomeSoundSample));
+  s = g_try_malloc (sizeof (GnomeSoundSample));
   if (!s)
     {
       fclose (f);
@@ -235,7 +236,7 @@ gnome_sound_sample_load_wav(const char *file)
 	    fseek (f, len, SEEK_CUR);
 	  else
 	    {
-	      s->data = g_malloc (len);
+	      s->data = g_try_malloc (len);
 	      if (!s->data)
 		{
 		  fclose (f);
@@ -299,11 +300,12 @@ static gboolean
 use_sound (void)
 {
   if (gnome_sound_connection == -1){
-    if (esound_hostname){
+    if (esound_hostname || esound_hostname_null_ok){
       gnome_sound_connection = esd_open_sound (esound_hostname);
       if (gnome_sound_connection == -1){
 	g_free (esound_hostname);
 	esound_hostname = NULL;
+	esound_hostname_null_ok = FALSE;
 	return FALSE;
       }
     }
@@ -319,8 +321,8 @@ static GnomeSoundSample *
 gnome_sound_sample_load_audiofile(const char *file)
 {
   AFfilehandle in_file;
-  GnomeSoundSample *s;
-  int in_format, in_width, in_channels;
+  GnomeSoundSample *s = NULL;
+  int in_format = -1, in_width = -1, in_channels;
   double in_rate;
   int bytes_per_frame;
   AFframecount frame_count, frames_read;
@@ -343,7 +345,16 @@ gnome_sound_sample_load_audiofile(const char *file)
     out_bits = ESD_BITS16;
   else {
       g_warning ("only sample widths of 8 and 16 supported");
-      return NULL;
+      goto error_case;
+  }
+
+  if (frame_count == -1 ||
+      in_channels == -1 ||
+      in_rate == -1 ||
+      in_format == -1 ||
+      in_width == -1) {
+      g_warning ("Default track not found");
+      goto error_case;
   }
 
   bytes_per_frame = in_width / 8;
@@ -354,7 +365,7 @@ gnome_sound_sample_load_audiofile(const char *file)
     out_channels = ESD_STEREO;
   else {
       g_warning ("only 1 or 2 channel samples supported");
-      return NULL;
+      goto error_case;
   }
 
   out_format = out_bits | out_channels | out_mode | out_func;
@@ -366,12 +377,19 @@ gnome_sound_sample_load_audiofile(const char *file)
   s->rate = out_rate;
   s->format = out_format;
   s->samples = frame_count;
-  s->data = g_malloc(frame_count * in_channels * bytes_per_frame);
+  s->data = g_try_malloc(frame_count * in_channels * bytes_per_frame);
   s->id = 0;
 
+  if (!s->data) {
+    g_free (s);
+    s = NULL;
+    goto error_case;
+  }
+  
   frames_read = afReadFrames(in_file, AF_DEFAULT_TRACK, s->data,
 			     frame_count * in_channels);
 
+ error_case:
   afCloseFile(in_file);
 
   return s;
@@ -381,12 +399,15 @@ gnome_sound_sample_load_audiofile(const char *file)
 
 /**
  * gnome_sound_sample_load:
- * @sample_name: the name of the sample
- * @filename: the filename where the audio is stored
+ * @sample_name: The name of the sample.
+ * @filename: The filename where the audio is stored.
  *
- * Loads the audio on @filename and XXXX
+ * Loads the audio from @filename and load it into the esd cache for later
+ * playing. Programs will rarely want to call this function directly. Use
+ * gnome_sound_play() instead for fire and forget sound playing.
  *
- * Returns: a sample_id, or a negative number otherwise.
+ * Returns: The esound sample_id or %-1 if the sample was unable to be cached
+ * for esound.
  */
 int
 gnome_sound_sample_load(const char *sample_name, const char *filename)
@@ -455,9 +476,10 @@ gnome_sound_sample_load(const char *sample_name, const char *filename)
 
 /**
  * gnome_sound_play:
- * @filename: file containing the sound sample
+ * @filename: File containing the sound sample.
  *
- * Plays the audio stored in @filename
+ * Plays the audio stored in @filename, if possible. Fail quietly if playing is
+ * not possible (due to missing sound support or for other reasons).
  */
 void 
 gnome_sound_play (const char * filename)
@@ -490,9 +512,9 @@ gnome_sound_play (const char * filename)
 
 /**
  * gnome_sound_init:
- * @hostname: hostname where esd daemon resides.
+ * @hostname: Hostname where esd daemon resides.
  *
- * Initialize esd connection
+ * Initialize the esd connection.
  */
 void
 gnome_sound_init(const char *hostname)
@@ -500,14 +522,17 @@ gnome_sound_init(const char *hostname)
 #ifdef HAVE_ESD
 	srand(time(NULL));
 	g_free (esound_hostname);
-	esound_hostname = g_strdup (hostname);
+	if (hostname)
+		esound_hostname = g_strdup (hostname);
+	else
+		esound_hostname_null_ok = TRUE;
 #endif
 }
 
 /** 
  * gnome_sound_shutdown:
  *
- * shuts down the gnome sound support
+ * Shuts down the gnome sound support.
  */
 void
 gnome_sound_shutdown(void)
@@ -525,8 +550,12 @@ gnome_sound_shutdown(void)
 /** 
  * gnome_sound_connection_get:
  *
- * Returns: the fd of our Esound connection or -1
- * on error
+ * Rarely needed to by programs directly, this function may be useful if a
+ * program has cached a sample with gnome_sound_sample_load() and now wishes to
+ * call esd_sample_play() to play the sample.
+ *
+ * Returns: the file descriptor of our esound connection or %-1
+ * on error.
  **/
 int
 gnome_sound_connection_get (void)
